import numpy


class Perceptron:

    def __init__(self, start_weights: list, threshold: float, transfer_function):
        # Das erste Gewicht ist -threshold. Hierdurch muss der Threshold nicht bei jeder Summe mitberechnet werden
        self.weights = [- threshold] + start_weights
        self.transfer_function = transfer_function

    def classify(self, features: list):
        """
        Klassifiziert einen Vektor an Merkmalen.
        Die Dimension des Merkmalsvektors muss dabei mt der Dimension des Startgewichtvektors übereinstimmen.
        """

        if len(features) != len(self.weights) - 1:
            print("Es wurden nicht genug Features übergeben")
            return

        # Das erste Merkmal m_0 ist konstant 1
        all_features = [1] + features
        transposed_weights = numpy.transpose(self.weights)

        # Multipliziere die transponierten mit dem Merkmalsvektor
        result = numpy.matmul(transposed_weights, all_features)

        # Wende die Aktivierungsfunktion auf das Ergebnis an
        return self.transfer_function(result)
