import math
import random

from algorithm.pla.perceptron_learning_algorithm import train, train_pocket
from algorithm.pla.perceptron import Perceptron
from algorithm.pla.transfer_functions import normalized_tanh
from aufgaben.p6.evaluate import evaluate
from korpus import create_bewegung


def apply_pla():
    weights = [random.random(), random.random(), random.random()]
    threshold = -1
    perceptron = Perceptron(weights, threshold, normalized_tanh)
    joggen = create_bewegung('joggen').messungen[2]
    kniebeuge = create_bewegung('kniebeuge').messungen[2]
    training_data = [
        [[joggen.sensoren[0].werte[0], joggen.sensoren[1].werte[0], joggen.sensoren[2].werte[0]], 1],
        [[kniebeuge.sensoren[0].werte[17], kniebeuge.sensoren[1].werte[17], kniebeuge.sensoren[2].werte[17]], 0],
        [[joggen.sensoren[0].werte[17], joggen.sensoren[1].werte[17], joggen.sensoren[2].werte[17]], 1],
        [[joggen.sensoren[0].werte[4], joggen.sensoren[1].werte[4], joggen.sensoren[2].werte[4]], 1],
        [[kniebeuge.sensoren[0].werte[9], kniebeuge.sensoren[1].werte[9], kniebeuge.sensoren[2].werte[9]], 0],
        [[kniebeuge.sensoren[0].werte[104], kniebeuge.sensoren[1].werte[104], kniebeuge.sensoren[2].werte[104]], 0],
        [[joggen.sensoren[0].werte[104], joggen.sensoren[1].werte[104], joggen.sensoren[2].werte[104]], 1],
        [[kniebeuge.sensoren[0].werte[2], kniebeuge.sensoren[1].werte[2], kniebeuge.sensoren[2].werte[2]], 0],
        [[joggen.sensoren[0].werte[2], joggen.sensoren[1].werte[2], joggen.sensoren[2].werte[2]], 1],
        [[kniebeuge.sensoren[0].werte[25], kniebeuge.sensoren[1].werte[25], kniebeuge.sensoren[2].werte[25]], 0],
        [[joggen.sensoren[0].werte[25], joggen.sensoren[1].werte[25], joggen.sensoren[2].werte[25]], 1]
    ]
    max_iterations = 1000
    learning_rate = 0.1
    train(perceptron, training_data, max_iterations, learning_rate)
    print("\nPLA: Zunächst Gewichte, dann Klassifizierung. Gehen = 1, Kniebeuge = 0")
    print(perceptron.weights)
    amount_of_weights = 3
    correct_data = []
    incorrect_data = []
    for i in range(amount_of_weights):
        correct_data.append(joggen.sensoren[i].werte)
        incorrect_data.append(kniebeuge.sensoren[i].werte)
    evaluate(500, amount_of_weights, correct_data, incorrect_data, perceptron)



def apply_pocket():
    weights = [0.5, 0.2, 0.7]
    threshold = -1
    perceptron = Perceptron(weights, threshold, normalized_tanh)
    joggen = create_bewegung('joggen').messungen[2]
    kniebeuge = create_bewegung('kniebeuge').messungen[2]
    training_data = [
        [[joggen.sensoren[0].werte[0], joggen.sensoren[1].werte[0], joggen.sensoren[2].werte[0]], 1],
        [[kniebeuge.sensoren[0].werte[17], kniebeuge.sensoren[1].werte[17], kniebeuge.sensoren[2].werte[17]], 0],
        [[joggen.sensoren[0].werte[17], joggen.sensoren[1].werte[17], joggen.sensoren[2].werte[17]], 1],
        [[joggen.sensoren[0].werte[4], joggen.sensoren[1].werte[4], joggen.sensoren[2].werte[4]], 1],
        [[kniebeuge.sensoren[0].werte[9], kniebeuge.sensoren[1].werte[9], kniebeuge.sensoren[2].werte[9]], 0],
        [[kniebeuge.sensoren[0].werte[104], kniebeuge.sensoren[1].werte[104], kniebeuge.sensoren[2].werte[104]], 0],
        [[joggen.sensoren[0].werte[104], joggen.sensoren[1].werte[104], joggen.sensoren[2].werte[104]], 1],
        [[kniebeuge.sensoren[0].werte[2], kniebeuge.sensoren[1].werte[2], kniebeuge.sensoren[2].werte[2]], 0],
        [[joggen.sensoren[0].werte[2], joggen.sensoren[1].werte[2], joggen.sensoren[2].werte[2]], 1],
        [[kniebeuge.sensoren[0].werte[25], kniebeuge.sensoren[1].werte[25], kniebeuge.sensoren[2].werte[25]], 0],
        [[joggen.sensoren[0].werte[25], joggen.sensoren[1].werte[25], joggen.sensoren[2].werte[25]], 1]
    ]
    max_iterations = 1
    learning_rate = 0.10
    train_pocket(perceptron, training_data, max_iterations, learning_rate)
    print("\nPocket: Zunächst Gewichte, dann Klassifizierung. Gehen = 1, Kniebeuge = 0")
    print(perceptron.weights)
    amount_of_weights = 3
    correct_data = []
    incorrect_data = []
    for i in range(amount_of_weights):
        correct_data.append(joggen.sensoren[i].werte)
        incorrect_data.append(kniebeuge.sensoren[i].werte)
    evaluate(500, amount_of_weights, correct_data, incorrect_data, perceptron)


if __name__ == '__main__':
    apply_pla()
    #apply_pocket()