import csv
import glob

from numpy import genfromtxt

from korpus.bewegung import Bewegung
from korpus.messung import Messung
from korpus.sensor import Sensor


def create_bewegung(name, columns=range(1, 10)):
    messungen = []
    anzahl = None
    zeit = None

    for index, file_name in enumerate(glob.glob(f"..\\..\\daten\\{name}\\*.csv")):
        data, messung = create_messungen(columns, file_name, index)

        messungen.append(messung)
        anzahl = len(data)
        zeit = anzahl / 100

    return Bewegung(name, messungen, anzahl, zeit)


def create_messungen(columns, file_name, index):
    sensor_names = get_sensor_names(file_name, columns)

    data = genfromtxt(file_name,
                      delimiter=',',
                      usecols=columns,
                      skip_header=5)
    sensoren = []
    for sensor_index in range(len(columns)):
        sensor = Sensor(sensor_names[sensor_index], data.transpose()[sensor_index])
        sensoren.append(sensor)

    messung = Messung(f"Sensor {index + 1}", sensoren)
    return data, messung


def get_sensor_names(file_name, columns):
    with open(file_name, newline='') as file:
        reader = csv.reader(file)

        # Skippe die 4 Header-Zeilen
        next(reader)
        next(reader)
        next(reader)
        next(reader)

        # Gib die Namen zurück, die ausgelesen werden sollen
        all_sensor_names = next(reader)
        return [all_sensor_names[i] for i in columns]
